"""AIDiscuss - AI-powered chat application with multi-provider support."""

__version__ = "0.1.0"
__author__ = "Devsarc"
__license__ = "MIT"

# Package metadata
__all__ = ["__version__", "__author__", "__license__"]
